

# python matrix_visualisation.py <job> <molecule>
# ver. 1.1


from scipy.io import mmread,mmwrite
import scipy.sparse as sp
import matplotlib.pyplot as plt
import matplotlib.colors as colors

from matplotlib.ticker import FuncFormatter,FixedLocator


import numpy as np
import itertools
from numpy import linalg as la

import configparser
import ast

import re

import sys,getopt





opts,args = getopt.getopt(sys.argv[1:],'b:')



print (opts)
#print args



commline = args
print ('arguments:',commline)




# INI file parsing
Config = configparser.ConfigParser()
Config.read("matrix_visualisation.ini")
print ('ini:',Config.sections())

def ConfigSectionMap(section):
    dict1 = {}
    options = Config.options(section)
    for option in options:
        try:
            dict1[option] = Config.get(section, option)
            if dict1[option] == -1:
                DebugPrint("skip: %s" % option)
        except:
            print("exception on %s!" % option)
            dict1[option] = None
    return dict1






# data loading
suffix = ConfigSectionMap("common")['suffix']
nazvy = np.array([ConfigSectionMap("common")['subblocks'].split(",")]).reshape(2,2)

print (nazvy)


nasel = 0
for zkus in range(len(opts)):
  if opts[zkus][0]=='-b':
    bblocks = opts[zkus][1].split(',')
    print (bblocks)
    blocks = [int(float(bblocks[zkus])/3.0) for zkus in range(len(bblocks))]
    nasel=1
    print ('blocks',blocks)








if nasel==0:
  vsechny = np.array([''])
else:
  vsechny = np.array(['','_1','_2','_offd','_offdT'])




for udelej in range(len(vsechny)):

  print (vsechny[udelej])

#  outname =  args[0]+'/'+args[1]
  outname =  args[0]+'_'+args[1]


  with open(outname + '_atoms.txt') as soub:
    for line in soub:
      atoms = line.strip().split(',')




  file = outname+'_forceconst'+vsechny[udelej]+suffix
  print (file)

  A = mmread(file)
  Fone = A.todense()
  rows = np.shape(Fone)[0]
  cols = np.shape(Fone)[1]

  print ('rows,cols =',rows,cols)

  print (np.shape(Fone))

  ratoms = atoms
  catoms = atoms



  if vsechny[udelej]!='':
    bj,bi =np.where(nazvy==vsechny[udelej]);

    print (bj,bi)

    ratoms = atoms[bj[0]*int(blocks[0]):int(blocks[0])+bj[0]*int(blocks[1]):1]
    catoms = atoms[bi[0]*int(blocks[0]):int(blocks[0])+bi[0]*int(blocks[1]):1]

  mini = np.min(np.min(Fone))
  maxi = np.max(np.max(Fone))

  oodkud = abs(mini)
  if abs(maxi)>oodkud:
     oodkud = abs(maxi)

  print ( mini,'..',maxi,'; ',oodkud)

  for i in range(rows):
     for j in range(cols):
        if Fone[i,j]>0:
          Fone[i,j] = np.log10(1.0+9.0*Fone[i,j]/oodkud)
        elif Fone[i,j]<0:
          Fone[i,j] = -np.log10(1.0+9.0*abs(Fone[i,j]/oodkud))


  cislo = oodkud
  oodkud = np.ceil(10**(np.log10(cislo)-round(np.log10(cislo))+1))*10**(round(np.log10(cislo)-1))
  print (cislo,'->',oodkud)


  mini = np.min(np.min(Fone))
  maxi = np.max(np.max(Fone))

  odkud = abs(mini)
  if abs(maxi)>odkud:
    odkud = abs(maxi)

  print (mini,'..',maxi,'; ',odkud)


  cmapa = colors.LinearSegmentedColormap.from_list("",["navy","blue","skyblue","white","salmon","red","maroon"])

  fig = plt.figure()
  ax = fig.add_subplot(111)
  cax = ax.matshow(Fone,cmap=cmapa,vmin=-odkud,vmax=odkud)
  cbar = fig.colorbar(cax)


  hits1 = [1.0 + 9.0/float(i) for i in range(1,11)]
  hits2 = [1.0 + 9.0/(10.0*float(i)) for i in range(2,11)]
  hits = np.concatenate((hits1,hits2))

  minors1 = 0.5*np.log10(hits)+0.5
  minors2 = -0.5*np.log10(hits)+0.5
  minors = np.concatenate((minors1,minors2))

  logs = [-1.0,-0.7403,-0.51188,-0.27875,0.0,0.27875,0.51188,0.7403,1.0]
  majors = 0.5*np.array(logs)+0.5

  cbar.ax.yaxis.set_major_locator(FixedLocator(majors))
  cbar.ax.yaxis.set_minor_locator(FixedLocator(minors))




  def millions(x,pos):
    logtrue = (x-0.5)/0.5*odkud
    if logtrue>=0:
      pomo = (10**abs(logtrue)-1.0)*oodkud/9.0
    else:
      pomo = -(10**abs(logtrue)-1.0)*oodkud/9.0

    return '%1.2f' % (pomo)

  formatter = FuncFormatter(millions)
  cbar.ax.yaxis.set_major_formatter(formatter)




  ax.yaxis.set_major_locator(plt.FixedLocator(np.arange(-0.5,rows,3)))
  ax.yaxis.set_major_formatter(plt.NullFormatter())

  ax.xaxis.set_major_locator(plt.FixedLocator(np.arange(-0.5,cols,3)))
  ax.xaxis.set_major_formatter(plt.NullFormatter())


  ax.tick_params(axis='x',colors=(0,0,0,0))
  ax.tick_params(axis='y',colors=(0,0,0,0))



  ax.grid()




  for i in range(int(cols/3.0)):
    ax.annotate(catoms[i],xy=(3*i,-1.0),annotation_clip=False)


    
  for i in range(int(rows/3.0)):
    match = re.search('.',ratoms[i])
    ax.annotate(match.group(0),xy=(-3.0,3*i+2.0),annotation_clip=False)

  ax.annotate(r'${\bf F}$[Ha/Bohr$^2$]',xy=(33,7.5))



  plt.savefig(outname+vsechny[udelej]+'.pdf');plt.savefig(outname+vsechny[udelej]+'.png')








